import Flutter
import UIKit

public class SwiftVerifaiSdkPlugin: NSObject, FlutterPlugin, CoreApi, NfcApi, LivenessApi {
    
    static var coreResultApi: CoreResultApi?
    static var nfcResultApi: NfcResultApi?
    static var livenessResultApi: LivenessResultApi?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let messenger : FlutterBinaryMessenger = registrar.messenger()
        let api : CoreApi & LivenessApi & NfcApi & NSObjectProtocol = SwiftVerifaiSdkPlugin()
        // API setup
        CoreApiSetup(messenger, api);
        LivenessApiSetup(messenger, api)
        NfcApiSetup(messenger, api)
        // Result handlers
        coreResultApi = CoreResultApi(binaryMessenger: messenger)
        nfcResultApi = NfcResultApi(binaryMessenger: messenger)
        livenessResultApi = LivenessResultApi(binaryMessenger: messenger)
    }
    
    // MARK: - Core
    public func setLicenceLicence(_ licence: String,
                                  error: AutoreleasingUnsafeMutablePointer<FlutterError?>) {
        Core.setLicence(licence, resultHandler: SwiftVerifaiSdkPlugin.coreResultApi)
    }
    
    public func configureConfiguration(_ configuration: Configuration,
                                       error: AutoreleasingUnsafeMutablePointer<FlutterError?>) {
        Core.processConfiguration(configuration, resultHandler: SwiftVerifaiSdkPlugin.coreResultApi)
    }
    
    public func startWithError(_ error: AutoreleasingUnsafeMutablePointer<FlutterError?>) {
        Core.start(rootViewController: SwiftVerifaiSdkPlugin.rootFlutterViewController(),
                   resultHandler: SwiftVerifaiSdkPlugin.coreResultApi)
    }
    
    // MARK: - NFC
    public func startConfig(_ config: NfcConfiguration,
                            coreResult: CoreResult,
                            error: AutoreleasingUnsafeMutablePointer<FlutterError?>) {
        NFC.start(rootViewController: SwiftVerifaiSdkPlugin.rootFlutterViewController(),
                  configuration: config,
                  resultHandler: SwiftVerifaiSdkPlugin.nfcResultApi)
    }
    
    // MARK: - Liveness
    public func startConfig(_ config: LivenessConfiguration,
                            error: AutoreleasingUnsafeMutablePointer<FlutterError?>) {
        Liveness.start(rootViewController: SwiftVerifaiSdkPlugin.rootFlutterViewController(),
                       configuration: config,
                       resultHandler: SwiftVerifaiSdkPlugin.livenessResultApi)
    }
    
    // MARK: - Supporting Functions
    static func rootFlutterViewController() -> FlutterViewController? {
        if let delegate = UIApplication.shared.delegate as? FlutterAppDelegate,
            let rootVC = delegate.window.rootViewController as? FlutterViewController {
            return rootVC
        }
        return nil
    }
}
