import 'package:pigeon/pigeon.dart';

// pigeon configuration
@ConfigurePigeon(PigeonOptions(
    dartOut: 'lib/verifai_sdk.dart',
    javaOut: 'android/src/main/java/com/verifai/verifai_sdk/Messages.java',
    javaOptions: JavaOptions(package: 'com.verifai.verifai_sdk'),
    objcHeaderOut: 'ios/Classes/Messages.h',
    objcSourceOut: 'ios/Classes/Messages.m'))
class MrzDataResult {
  String? mrzString;
  String? format;
  String? documentType;
  String? documentSubType;
  String? countryCode;
  String? lastName;
  String? firstName;
  String? documentNumber;
  String? documentNumberCheckDigit;
  String? nationality;
  String? dateOfBirthRaw;
  String? dateOfBirthCheckDigit;
  String? sex;
  String? dateOfExpiryRaw;
  String? dateOfExpiryCheckDigit;
  String? optionalData;
  String? optionalDataCheckDigit;
  String? optionalDataTwo;
  String? compositeCheckDigit;
  Map<String?, String?>? documentSpecificData;
  String? nfcKey;
  double? validityScore;
  bool? isNfcKeyValid;
  bool? isDocumentCodeValid;
  bool? isDocumentNumberValid;
  bool? isDateOfBirthValid;
  bool? isDateOfExpiryValid;
  bool? isCompositeValid;
  bool? isOptionalDataValid;
  bool? isLine1LengthValid;
  bool? isLine2LengthValid;
  bool? isLine3LengthValid;
}

class CoordinatePoint {
  late int? x;
  late int? y;
}

class Polygon {
  CoordinatePoint? topLeft;
  CoordinatePoint? topRight;
  CoordinatePoint? bottomRight;
  CoordinatePoint? bottomLeft;
}

class DocumentZoneResult {
  String? title;
  String? side;
  double? x;
  double? y;
  double? width;
  double? height;
  bool? block;
}

class DocumentResult {
  String? uuid;
  String? type;
  String? model;
  String? country;
  double? widthMm;
  double? heightMm;
  String? sampleFront;
  String? mrzType;
  String? mrzStart;
  String? nfcType;
  List<DocumentZoneResult?>? zones;
  int? numberOfSides;
  bool? hasMrz;
}

enum BarcodeSymbology {
  qr,
  code39,
  code128,
  pdf417,
}

enum DocumentSide { front, back }

class BarcodeResult {
  BarcodeSymbology? symbology;
  DocumentSide? side;
  String? data;
}

class CoreResult {
  Uint8List? frontImage;
  Polygon? frontDocumentPosition;
  Uint8List? backImage;
  Polygon? backDocumentPosition;
  MrzDataResult? mrzData;
  DocumentResult? document;
  List<BarcodeResult?>? documentBarcodes;
  Map<String?, String?>? visualInspectionZoneResult;
}

enum DocumentType {
  idCard,
  driversLicence,
  passport,
  refugee,
  emergencyPassport,
  residencePermitTypeI,
  residencePermitTypeII,
  visa,
  unknown
}

enum ValidatorType {
  countryAllowlist,
  countryBlocklist,
  hasMrz,
  documentTypes,
  mrzAvailable,
  nfcKeyWhenAvailable,
}

class Validator {
  ValidatorType? type;
  Map<String?, Object?>? args;
}

enum DocumentFilterType {
  documentTypeAllowList,
  documentAllowList,
  documentBlockList,
}

class DocumentFilter {
  DocumentFilterType? type;
  Map<String?, Object?>? args;
}

enum InstructionScreenId {
  documentChooserTabBarInstruction,
  mrzPresentFlowInstruction,
  mrzScanFlowInstruction
}

enum InstructionScreenType {
  media,
  hidden,
  defaultType, // default is a keyword
  web,
}

class InstructionScreen {
  InstructionScreenId? screen;
  InstructionScreenType? type;
  String? title;
  String? continueButtonLabel;
  String? header;
  String? mp4FileName;
  String? instruction;
  String? url;
}

class InstructionScreenConfiguration {
  bool? showInstructionScreens;
  List<InstructionScreen?>? instructionScreens;
}

// Only available for iOS
class ScanHelpConfiguration {
  bool? showScanHelp;
  String? customScanHelpScreenInstructions;
  String? customScanHelpScreenMp4FileName;
}

class Configuration {
  bool? requireDocumentCopy;
  bool? enablePostCropping;
  bool? enableManual;
  bool? requireMrzContents;
  bool? requireNfcWhenAvailable;
  bool? readMrzContents;
  List<Validator?>? validators;
  List<DocumentFilter?>? documentFilters;
  bool? documentFiltersAutoCreateValidators;
  String? customDismissButtonTitle; // Only available on iOS
  bool? isScanHelpEnabled; // Only available on iOS
  bool? requireCroppedImage;
  InstructionScreenConfiguration?
      instructionScreenConfiguration; // Causes crash, bug in Pigeon???: https://github.com/flutter/flutter/issues/99150
  ScanHelpConfiguration? scanHelpConfiguration;
  bool? enableVisualInspection;
}

@FlutterApi()
abstract class CoreResultApi {
  void onSuccess(CoreResult result);
  void onCancelled();
  void onError(String message);
}

@HostApi()
abstract class CoreApi {
  void setLicence(String licence);
  void start();
  void configure(Configuration configuration);
}

//----- NFC -----

enum BacStatus {
  success,
  failed,
  notChecked,
  randomFailed,
}

enum AaStatus {
  success,
  signatureFailed,
  failed,
  notPresent,
  notChecked,
}

enum RootCertificateStatus {
  notFound,
  notChecked,
  valid,
  invalid,
  revoked,
}

enum CaStatus {
  success,
  failed,
  notPresent,
  notChecked,
}

enum ReadStatus {
  noAccess,
  fileNotFound,
  success,
  failed,
}

class LdsFile {
  Uint8List? data;
  Uint8List? dataHash;
  ReadStatus? readStatus;
}

// Only available for Android
class X509Certificate {
  int? version;
  int? serialNumber;
  String? signatureAlgorithmOID;
  String? signatureAlgorithmName;
  Uint8List? signatureAlgorithmParams;
  String? issuerName;
  String? notBefore;
  String? notAfter;
  String? subjectName;
  Uint8List? tbsCertificate;
  Uint8List? signature;
  List<bool?>? issuerUniqueID;
  List<bool?>? subjectUniqueID;
  List<bool?>? keyUsage;
  List<String?>? extendedKeyUsage;
}

class NfcResult {
  bool? originality;
  bool? authenticity;
  bool? confidentiality;
  bool? mrzMatch;
  bool? comSodMatch;
  BacStatus? bacStatus;
  AaStatus? activeAuthenticationStatus;
  bool? documentCertificateValid;
  RootCertificateStatus? signingCertificateMatchesWithParent;
  bool? scanCompleted;
  CaStatus? chipAuthenticationStatus;
  bool? documentSignatureCorrect;
  Uint8List? photo;
  MrzDataResult? mrzData;
  Map<int?, LdsFile?>? dataGroups;
  X509Certificate? documentCertificate; // Only available on Android
  Uint8List? iOSDocumentCertificate; // Only available on iOS
  Map<int?, Uint8List?>? sodHashes;
  Uint8List? sodData;
  int? featurePoints; // Only available on Android
  Uint8List? mimeBytes; // Only available on Android
  String? aaDigestAlgorithm;
  String? chipAuthenticationOid;
  String? chipAuthenticationAgreementAlgorithm; // Only available on Android
  String? chipAuthenticationPublicKeyAlgorithm; // Only available on Android
  X509Certificate? signingCertificate; // Only available on Android
  Uint8List? iOSSigningCertificate; // Only available on iOS
}

class NfcConfiguration {
  bool? retrieveImage;
  bool? showDismissButton; // Only available on iOS
  String? customDismissButtonTitle; // Only available on iOS
  bool? showInstructionScreens;
  NfcInstructionScreenConfiguration? instructionScreenConfiguration;
  ScanHelpConfiguration? scanHelpConfiguration; // Only available on iOS
}

class NfcInstructionScreenConfiguration {
  bool? showInstructionScreens;
  List<NfcInstructionScreen?>? instructionScreens;
}

enum NfcInstructionScreenId { nfcScanFlowInstruction }

class NfcInstructionScreen {
  NfcInstructionScreenId? screen;
  InstructionScreenType? type;
  String? title;
  String? continueButtonLabel;
  String? header;
  String? mp4FileName;
  String? instruction;
  String? url;
}

@FlutterApi()
abstract class NfcResultApi {
  void onSuccess(NfcResult result);
  void onCancelled();
  void onError(String message);
}

@HostApi()
abstract class NfcApi {
  void start(NfcConfiguration config, CoreResult coreResult);
}

//----- Liveness -----

enum LivenessCheckStatus {
  success,
  undetermined,
  skipped,
  notStarted,
  failed,
}

class LivenessCheckResult {
  String? name;
  String? instruction;
  LivenessCheckStatus? status;
  String? recordingFileName; // Only available on iOS
  Map<String?, Object?>? checkSpecificData;
}

class LivenessResult {
  bool? automaticChecksPassed;
  double? successRatio;
  List<LivenessCheckResult?>? resultList;
}

enum LivenessCheckType {
  closeEyes,
  tilt,
  speech,
  faceMatching,
}

class LivenessCheck {
  LivenessCheckType? type; // Make 'late' doesn't work with pigeon for now
  Map<String?, Object?>? args;
}

class LivenessConfiguration {
  bool? showDismissButton; // Only available on iOS
  List<LivenessCheck?>? checks;
  String? resultOutputDirectory;
  String? customDismissButtonTitle; // Only available on iOS
}

@FlutterApi()
abstract class LivenessResultApi {
  void onSuccess(LivenessResult result);
  void onError(String message);
}

@HostApi()
abstract class LivenessApi {
  void start(LivenessConfiguration config);
}
